@@ -1,33 +1,42 @@
 package Beam::Wire;
 # ABSTRACT: Lightweight Dependency Injection Container
-$Beam::Wire::VERSION = '1.010';
+$Beam::Wire::VERSION = '1.011';
 use strict;
 use warnings;
 
+use Scalar::Util qw( blessed );
 use Moo;
 use Config::Any;
-use Class::Load qw( load_class );
+use Module::Runtime qw( use_module );
 use Data::DPath qw ( dpath );
+use Path::Tiny qw( path );
 use File::Basename qw( dirname );
-use File::Spec::Functions qw( splitpath catfile file_name_is_absolute );
-use MooX::Types::MooseLike::Base qw( :all );
-use List::MoreUtils qw( all );
+use Types::Standard qw( :all );
 
 
 
 has file => (
     is      => 'ro',
-    isa     => Str,
+    isa     => InstanceOf['Path::Tiny'],
+    coerce => sub {
+        if ( !blessed $_[0] || !$_[0]->isa('Path::Tiny') ) {
+            return path( $_[0] );
+        }
+        return $_[0];
+    },
 );
 
 
 has dir => (
     is      => 'ro',
-    isa     => Str,
+    isa     => InstanceOf['Path::Tiny'],
     lazy    => 1,
-    default => sub {
-        my ( $volume, $path, $file ) = splitpath( $_[0]->file );
-        return join "", $volume, $path;
+    default => sub { $_[0]->file->parent },
+    coerce => sub {
+        if ( !blessed $_[0] || !$_[0]->isa('Path::Tiny') ) {
+            return path( $_[0] );
+        }
+        return $_[0];
     },
 );
 
@@ -42,11 +51,7 @@ has config => (
 sub _build_config {
     my ( $self ) = @_;
     return {} if ( !$self->file );
-    local $Config::Any::YAML::NO_YAML_XS_WARNING = 1;
-    my $loader = Config::Any->load_files( {
-        files  => [$self->file], use_ext => 1, flatten_to_hash => 1
-    } );
-    return "HASH" eq ref $loader ? (values(%{$loader}))[0] : {};
+    return $self->_load_config( $self->file );
 }
 
 
@@ -158,32 +163,41 @@ sub parse_args {
     return if not $args;
     my @args;
     if ( ref $args eq 'ARRAY' ) {
-        @args = @{$args};
+        @args = $self->find_refs( @{$args} );
     }
     elsif ( ref $args eq 'HASH' ) {
-        @args = %{$args};
+        # Hash args could be a ref
+        # Subcontainers cannot scan for refs in their configs
+        if ( $class->isa( 'Beam::Wire' ) ) {
+            my %args = %{$args};
+            my $config = delete $args{config};
+            # Relative subcontainer files should be from the current
+            # container's directory
+            if ( exists $args{file} && !path( $args{file} )->is_absolute ) {
+                $args{file} = $self->dir->child( $args{file} );
+            }
+            @args = $self->find_refs( %args );
+            if ( $config ) {
+                push @args, config => $config;
+            }
+        }
+        else {
+            my ( $maybe_ref ) = $self->find_refs( $args );
+            if ( blessed $maybe_ref ) {
+                @args = ( $maybe_ref );
+            }
+            else {
+                @args   = ref $maybe_ref eq 'HASH' ? %$maybe_ref
+                        : ref $maybe_ref eq 'ARRAY' ? @$maybe_ref
+                        : ( $maybe_ref );
+            }
+        }
     }
     else {
         # Try anyway?
         @args = $args;
     }
-    if ( $class->isa( 'Beam::Wire' ) ) {
-        my %args = @args;
-        # Subcontainers cannot scan for refs in their configs
-        my $config = delete $args{config};
-        # Relative subcontainer files should be from the current
-        # container's directory
-        if ( exists $args{file} && !file_name_is_absolute( $args{file} ) ) {
-            $args{file} = catfile( $self->dir, $args{file} );
-        }
-        @args = $self->find_refs( %args );
-        if ( $config ) {
-            push @args, config => $config;
-        }
-    }
-    else {
-        @args = $self->find_refs( @args );
-    }
+
     return @args;
 }
 
@@ -200,12 +214,22 @@ sub create_service {
         Beam::Wire::Exception::InvalidConfig->throw(
             name => $name,
             file => $self->file,
+            error => '"value" cannot be used with "class" or "extends"',
         );
     }
     if ( $service_info{value} ) {
         return $service_info{value};
     }
-    load_class( $service_info{class} );
+
+    if ( $service_info{config} ) {
+        my $conf_path = path( $service_info{config} );
+        if ( $self->file ) {
+            $conf_path = path( $self->file )->parent->child( $conf_path );
+        }
+        return $self->_load_config( "$conf_path" );
+    }
+
+    use_module( $service_info{class} );
     my $method = $service_info{method} || "new";
     my $service;
     if ( ref $method eq 'ARRAY' ) {
@@ -224,6 +248,37 @@ sub create_service {
         my @args = $self->parse_args( @service_info{"class","args"} );
         $service = $service_info{class}->$method( @args );
     }
+
+    if ( $service_info{on} ) {
+        my %meta = $self->get_meta_names;
+        my @listeners;
+
+        if ( ref $service_info{on} eq 'ARRAY' ) {
+            @listeners = map { [ %$_ ] } @{ $service_info{on} };
+        }
+        elsif ( ref $service_info{on} eq 'HASH' ) {
+            for my $event ( keys %{ $service_info{on} } ) {
+                if ( ref $service_info{on}{$event} eq 'ARRAY' ) {
+                    push @listeners,
+                        map {; [ $event => $_ ] }
+                        @{ $service_info{on}{$event} };
+                }
+                else {
+                    push @listeners, [ $event => $service_info{on}{$event} ];
+                }
+            }
+        }
+
+        for my $listener ( @listeners ) {
+            my ( $event, $conf ) = @$listener;
+            # XXX: Make $args prepend arguments to the listener
+            # XXX: Make $args also resolve refs
+            my $method = delete $conf->{ $meta{method} };
+            my ( $listen_svc ) = $self->find_refs( $conf );
+            $service->on( $event => sub { $listen_svc->$method( @_ ) } );
+        }
+    }
+
     return $service;
 }
 
@@ -289,7 +344,7 @@ sub find_refs {
 sub is_meta {
     my ( $self, $arg ) = @_;
     my $prefix = $self->meta_prefix;
-    return all { /^\Q$prefix/ } keys %{$arg};
+    return !grep { !/^\Q$prefix/ } keys %{$arg};
 }
 
 sub get_meta_names {
@@ -338,6 +393,15 @@ sub resolve_ref {
 
 sub BUILD {
     my ( $self ) = @_;
+
+    if ( $self->file && !path( $self->file )->exists ) {
+        my $file = $self->file;
+        Beam::Wire::Exception::Constructor->throw(
+            attr => 'file',
+            error => qq{Container file '$file' does not exist},
+        );
+    }
+
     # Create all the eager services
     for my $key ( keys %{ $self->config } ) {
         my $config = $self->config->{$key};
@@ -348,17 +412,47 @@ sub BUILD {
     return;
 }
 
+# Load a config file
+sub _load_config {
+    my ( $self, $path ) = @_;
+    local $Config::Any::YAML::NO_YAML_XS_WARNING = 1;
+    my $loader = Config::Any->load_files( {
+        files  => [$path], use_ext => 1, flatten_to_hash => 1
+    } );
+   return "HASH" eq ref $loader ? (values(%{$loader}))[0] : {};
+}
+
 
 package Beam::Wire::Exception;
-$Beam::Wire::Exception::VERSION = '1.010';
+$Beam::Wire::Exception::VERSION = '1.011';
 use Moo;
 with 'Throwable';
+use Types::Standard qw( :all );
+use overload q{""} => sub { $_[0]->error };
+
+has error => (
+    is => 'ro',
+    isa => Str,
+);
+
+
+package Beam::Wire::Exception::Constructor;
+$Beam::Wire::Exception::Constructor::VERSION = '1.011';
+use Moo;
+use Types::Standard qw( :all );
+extends 'Beam::Wire::Exception';
+
+has attr => (
+    is => 'ro',
+    isa => Str,
+    required => 1,
+);
 
 
 package Beam::Wire::Exception::Service;
-$Beam::Wire::Exception::Service::VERSION = '1.010';
+$Beam::Wire::Exception::Service::VERSION = '1.011';
 use Moo;
-use MooX::Types::MooseLike::Base qw( :all );
+use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
 
 has name => (
@@ -369,20 +463,40 @@ has name => (
 
 has file => (
     is          => 'ro',
-    isa         => Maybe[Str],
+    isa         => Maybe[InstanceOf['Path::Tiny']],
 );
 
 
 package Beam::Wire::Exception::NotFound;
-$Beam::Wire::Exception::NotFound::VERSION = '1.010';
+$Beam::Wire::Exception::NotFound::VERSION = '1.011';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 
+has '+error' => (
+    lazy => 1,
+    default => sub {
+        my ( $self ) = @_;
+        my $name = $self->name;
+        my $file = $self->file;
+        return "Service '$name' not found" . ( $file ? " in file '$file'" : '' );
+    },
+);
+
 
 package Beam::Wire::Exception::InvalidConfig;
-$Beam::Wire::Exception::InvalidConfig::VERSION = '1.010';
+$Beam::Wire::Exception::InvalidConfig::VERSION = '1.011';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
+use overload q{""} => sub {
+    my ( $self ) = @_;
+    my $file = $self->file;
+
+    sprintf "Invalid config for service '%s': %s%s",
+        $self->name,
+        $self->error,
+        ( $file ? " in file '$file'" : "" ),
+        ;
+};
 
 1;
 
@@ -396,7 +510,7 @@ Beam::Wire - Lightweight Dependency Injection Container
 
 =head1 VERSION
 
-version 1.010
+version 1.011
 
 =head1 SYNOPSIS
 
@@ -631,6 +745,119 @@ An C<eager> value will be created as soon as the container is created. If you
 have an object that registers itself upon instantiation, you can make sure your
 object is created as soon as possible by doing C<lifecycle: eager>.
 
+=head4 on
+
+Attach event listeners using L<Beam::Emitter|Beam::Emitter>.
+
+    emitter:
+        class: My::Emitter
+        on:
+            before_my_event:
+                $ref: listener
+                $method: on_before_my_event
+            my_event:
+                - $ref: listener
+                  $method: on_my_event
+                - $ref: other_listener
+                  $method: on_my_event
+    listener:
+        class: My::Listener
+    other_listener:
+        class: My::Listener
+
+Now, when the C<emitter> fires off its events, they are dispatched to the
+appropriate listeners.
+
+In order to work around a bug in YAML.pm, you can also specify event listeners
+as an array of hashes:
+
+    emitter:
+        class: My::Emitter
+        on:
+            - before_my_event:
+                $ref: listener
+                $method: on_before_my_event
+            - my_event:
+                $ref: listener
+                $method: on_my_event
+            - my_event:
+                $ref: other_listener
+                $method: on_my_event
+
+=head3 Config Services
+
+A config service allows you to read a config file and use it as a service, giving
+all or part of it to other objects in your container.
+
+To create a config service, use the C<config> key. The value is the path to the
+file to read. By default, YAML, JSON, XML, and Perl files are supported (via
+L<Config::Any>).
+
+    # db_config.yml
+    dsn: 'dbi:mysql:dbname'
+    user: 'mysql'
+    pass: '12345'
+
+    # container.yml
+    db_config:
+        config: db_config.yml
+
+You can pass in the entire config to an object using C<$ref>:
+
+    # container.yml
+    db_config:
+        config: db_config.yml
+    dbobj:
+        class: My::DB
+        args:
+            conf:
+                $ref: db_config
+
+If you only need the config file once, you can create an anonymous config
+object.
+
+    # container.yml
+    dbobj:
+        class: My::DB
+        args:
+            conf:
+                $config: db_config.yml
+
+The config file can be used as all the arguments to the service:
+
+    # container.yml
+    dbobj:
+        class: My::DB
+        args:
+            $config: db_config.yml
+
+In this example, the constructor will be called like:
+
+    my $dbobj = My::DB->new(
+        dsn => 'dbi:mysql:dbname',
+        user => 'mysql',
+        pass => '12345',
+    );
+
+You can reference individual items in a configuration hash using C<$path>
+references:
+
+    # container.yml
+    db_config:
+        config: db_config.yml
+    dbh:
+        class: DBI
+        method: connect
+        args:
+            - $ref: db_config
+              $path: /dsn
+            - $ref: db_config
+              $path: /user
+            - $ref: db_config
+              $path: /pass
+
+B<NOTE:> You cannot use C<$path> and anonymous config objects.
+
 =head3 Inner Containers
 
 Beam::Wire objects can hold other Beam::Wire objects!
@@ -804,6 +1031,10 @@ through unaltered.
 
 The base exception class
 
+=head2 Beam::Wire::Exception::Constructor
+
+An exception creating a Beam::Wire object
+
 =head2 Beam::Wire::Exception::Service
 
 An exception with service information inside
